SystemOrganization addCategory: #'Installer-Core'!

Object subclass: #Installer
	instanceVariableNames: 'sm wsm mc ma url root command markers project package bug desc answers packages messagesToSuppress pageDataStream date universe afile'
	classVariableNames: 'Remembered SkipLoadingTests IsSetToTrapErrors Entities Fixes WebSearchPath ValidationBlock LastUniUpdate InstallerBindings'
	poolDictionaries: ''
	category: 'Installer-Core'!

!Installer commentStamp: 'kph 7/26/2007 12:57' prior: 0!
This documentation also appears at: http://wiki.squeak.org/squeak/Installer
===
Installer provides a simple Domain Specific Language for installing packages from monticello and squeakmap of various version.

One design decision is to be able to paste scripts a workspace and run and tested from there without any special editing.

also recommended: Launcher and Scripter.

Installer can also be used together with Launcher as a squeak startup document with various commandline parameters which install packages and optionally save the image.
 
Scripter from http://www.squeaksource.com can be used to tidy up extra windows that scripts open when preparing an image for publication.

===
!!!!Examples - Using SqueakMap

1) #squeakmap
=squeakmap := Installer squeakmap.
=squeakmap install: 'DynamicBindings'.
=squeakmap open. "opens the squeak map loader gui"

2) 
=Installer squeakmap install: 'DynamicBindings'.

3) Alternatively using which uses http (similar usage api to squeakmap)
Very useful in images which do not have a working Squeakmap packageloader (3.7)

=squeakmap := Installer websqueakmap.
=squeakmap package: 'DynamicBindings'; install.

4) Abbreviated instanciation
=squeakmap := Installer sm. "squeakmap"
=squeakmap := Installer wsm. "web-squeakmap"

5) Finding Packages by Name
=(Installer sm match: 'Labby*') explore.

6) Obtaining List of Package Versions
=(Installer sm package: 'Labby & Walker'; versions)  explore.  

7) Searching Packages
=Installer sm search: 'seaside'.
=Installer sm search: 'author:*Smith'. 
="fields available for searching: author: name:, summary:,description:"

8) Specify specific version of a package for installation
=Installer sm install: 'Labby(17)'.

!!!!Examples - Using #monticello (abbreviated form #mc)

Monticello repository options:

=Installer monticello http: aUrl.
=Installer monticello http: aUrl user: name password: secret.
=Installer monticello ftp: host directory: dir user: name password: secret.
=Installer monticello magma: host port: aport
=Installer monticello goods: host port: aport.
=Installer monticello directory: dir.

1)
=squeaksource := Installer monticello http: 'http://www.squeaksource.com'.
=seaside := squeaksource project: 'Seaside'.
=seaside install: 'Comet-lr.8';
=		install: 'Scriptaculous'.

2) Beign particular about which package versions/authors to load

=squeaksource := Installer monticello http: 'http://www.squeaksource.com'		
=squeaksource project: 'Seaside';
=			  install: #('Comet-lr' 'Comet-pmm');  "either of these"
=			  install: 'Scriptaculous-lr.148'. "specific version"

3) Supplying user/password.
=squeaksource := Installer monticello http: 'http://www.squeaksource.com'.
=squeaksource user: 'me'; password: 'asecret'.

4) Abbreviated instanciation for convenience.
=squeaksource := Installer squeaksource.
=or
=squeaksource := Installer ss.

!!!!Examples - install straight from a url		
1a)
=(Installer url: 'http://minnow.cc.gatech.edu/squeak/uploads/5889/MakeTestsGreen39.cs') install.
1b)
=Installer installUrl: 'http://minnow.cc.gatech.edu/squeak/uploads/5889/MakeTestsGreen39.cs'.

2) Install items from a web location
= | dir |
= dir := Installer url: 'http://minnow.cc.gatech.edu/squeak/uploads/5889/'.
= dir install: 'MakeTestsGreen39.cs'.

!!!!!!Embedded scripts
3)
Scripts embedded in html web pages are delimited by <code st>...</code st>
= | page |
= page := Installer url: 'http://wiki.squeak.org/squeak/742'.
= page install.

4)
specify your own delimeters for an embedded script.
= | page |
= page := Installer url: 'http://wiki.squeak.org/742'.
= page markers: 'beginning of script...end of script'.
= page install.

Note: Scripts embedded in html or a swiki page may need to escape some entities.
Supported entities are &amp; &gt; &lt; &star; &quot; (see Installer-c-#entities)

!!!!Examples - Using Mantis

n.b. wherever a bug number is used below you
can use a string that begins with the bug number. 
The remainder of the string is ignored, useful to annotate the purpose of the fix.

1 . Viewing a file uploaded to a mantis bug report

=Installer mantis bug: 4874 view: 'Join.4.cs'.

2. Installing a file uploaded to a mantis bug report
=Installer mantis bug: 4874 fix: 'Join.4.cs'.

3. as above, inform user if bug report has been updated since a given date 
=Installer mantix	bug: 4874 fix: 'JoinTest.1.cs' date: '12-18-06'. 
	
4. Install or view a fix for a given bug.

=Installer mantis viewBug: 4874.
=Installer mantis fixBug: 4874.

The fix script is published in a note added to the bug report page with the following syntax

= "fix begin"
=  Installer mantis bug: 474 fix: 'Join.4.cs'.
= "fix test"
=  Installer mantis bug: 474 fix: 'JoinTest.1.cs' date: '12-18-06'. 
= "fix end"

5. Install a fix for a given bug, ignoring all test code
(test code being delimeted by: "fix test" ... "fix end" )

= Installer mantis justFixBug: 474.
= or
= Installer mantis justFixBug: 474 date: '12-19-06'.

6. View the mantis report for a bug.

= Installer mantis bug: 474.

7. Open a ChangeList browser on the file attached to the bug.
=Installer mantis bug: 474 cl: 'file.cs'.
7b Open a workspace on file contents.
=Installer mantis bug: 474 retrieve: 'file.cs'.

8. Load bug fix if it has not already been loaded.
= Installer ensureFix: '474 Date and time improvements'.
= Installer ensureFixes: #(474 123).

!!!!Universes Support

=Installer universe install: 'a pkg'; install: 'b pkg(1.1)'; install.

!!!!Additional features

A) Viewing files in monticello snapshot browser.

Where you can #install:/#install you can also #view:/#view.

=(Installer ss project:'Installer') view: 'Installer-Core'.
=(Installer url: 'http://installer.pbwiki.com/f/Installer.st') view.

=Installer ss project: 'Installer'; view: 'Installer-Core'.
=Installer ss project: 'Installer'; browse: 'Installer-Core'.

B) Debug

=Installer debug "Open debugger on error"
=Installer noDebug "Log errors"

C) Installer validationBlock: [  ]

If you have a long script which goes wrong, providing a validation block can be useful to track down the problem. If supplied it is executed after each installation. A debugger will be opened if the validation block returns false.

D) #rememberAs

Add your own favourite repositories.
=Installer monticello http: 'http://gjallar.krampe.se'; rememberAs: #gjallar.

henceforth:
=Installer gjallar install: 'Q2'.

E) Automate user interactions: #answer:with:

=repo := Installer ss project: 'Seaside'.
=repo answer: '&star;config&star;' with: 'seaside';
=	answer: '&star;password&star;' with: 'admin';
=	install: 'Seaside2.8a1'
	
	







!

!Installer class methodsFor: 'instanciation' stamp: 'kph 12/16/2007 12:00'!
initialize
	
	WebSearchPath := OrderedCollection 
		with:  [ 'http://installer.pbwiki.com/*-', (Smalltalk version replaceAll: $. with: $:)  ,'-', Utilities authorInitialsPerSe  ,'?raw=bare' ]
		with:  [ 'http://installer.pbwiki.com/*-', ((Smalltalk version) replaceAll: $. with: $:),'?raw=bare' ]
		with:  'http://installer.pbwiki.com/*?raw=bare'
	 ! !

!Installer class methodsFor: 'accessing' stamp: 'kph 12/21/2006 02:08'!
cancelSkipLoadingTests

"sets a flag to un-ignore loading of the testing portion of scripts embedded in pages"
SkipLoadingTests := false.
 ! !

!Installer class methodsFor: 'accessing system' stamp: 'kph 2/14/2007 07:36'!
classProjectLauncher

^Smalltalk at: #ProjectLauncher ifAbsent: [ self error: 'ProjectLauncher not present' ]! !

!Installer class methodsFor: 'accessing system' stamp: 'kph 7/25/2007 13:09'!
classUGlobalInstaller

^Smalltalk at: #UGlobalInstaller  ifAbsent: [ self error: 'Universes code not present' ]! !

!Installer class methodsFor: 'accessing system' stamp: 'kph 7/25/2007 13:10'!
classUUniverse

^Smalltalk at: #UUniverse  ifAbsent: [ self error: 'Universes code not present' ]! !

!Installer class methodsFor: 'accessing system' stamp: 'kph 1/5/2007 02:52'!
classes

^ Smalltalk! !

!Installer class methodsFor: 'debug' stamp: 'kph 5/9/2007 23:05'!
debug

	IsSetToTrapErrors := false! !

!Installer class methodsFor: 'launcher support' stamp: 'kph 12/16/2007 11:24'!
do: webPageName

	| rs |

	self webSearchPath isEmpty ifTrue: [ ^self error: 'search path not set' ].
	
	rs := webPageName readStream.
	
	[ rs atEnd ] whileFalse: [ self install: (rs upTo: $;) ].
! !

!Installer class methodsFor: 'accessing system' stamp: 'kph 5/28/2007 05:11'!
doesNotUnderstand: aMessage

	^ self remembered at: aMessage selector ifAbsent: [ super doesNotUnderstand: aMessage ]! !

!Installer class methodsFor: 'accessing' stamp: 'kph 12/23/2006 05:46'!
entities

^ Entities ifNil: [ Entities := 
"enough entities to be going on with"
  Dictionary new.
Entities at: 'lt' put: '<';
	at: 'gt' put: '>';
	at: 'amp' put: '&';
	at: 'star' put: '*';
	at: 'quot' put: '"';
	at: 'nbsp' put: ' ';
 	yourself
]

 ! !

!Installer class methodsFor: 'instanciation' stamp: 'kph 5/24/2007 19:51'!
file: fileName
 
^ (self new file: fileName) 
! !

!Installer class methodsFor: 'accessing' stamp: 'kph 1/7/2007 16:13'!
fixesApplied

^ Fixes ifNil: [ Fixes := OrderedCollection new ].! !

!Installer class methodsFor: 'documentation' stamp: 'kph 7/25/2007 13:25'!
history

"
7 Jan 2007  
!!Installer fixBug: <aBugNo>

aBugNo can now be a number or a string, beginning with a number. 
This allows the mantis bug report summary to be used verbatim.
It also provides more infomarion for Installer to support self documentation.

!!Install fix if not already installed
 Installer ensureFix: <aBugNoOrString>
 Installer ensureFixes: #(1 2 3 4)

Installer now keeps a list of fix <aBugNoOrString> that have been installed up to this point.
#ensureFix: will only install the fix if it has not already been loaded.
note that only the bugNumber not the description is significant in the check.

8 Jan 2007
!!Installer view: <webPageNameOrUrl>

Provided that web page based scripts follow some simple rules, installer can collate the scripts from 
web pages into a single workspace where you can manually 'doit' portions as you wish.

The report generation is not very clever, it only matches on:
 'Installer install:' ,  'Installer installUrl:', and 'Installer mantis fixBug:'
 note these lines must be properly completed with an ending $. (period).

also invoked by commandline option VIEW=<webPageNameOrUrl>

10 Jan 2007
!!Now matches simpler html

Check for an html page, now matches
'<!!DOCTYPE HTML' and <html> 
the allows use of pbwiki's raw=bare option which returns iframe 
embeddable html without the usual headers.

8 May 2007
Modified bug:fix:date: so that the fixesApplied history does not contain unnecessary duplicate entries.
Fixed changeset naming for mantis bugs.

25 July 2007
Added Universes  Support
"! !

!Installer class methodsFor: 'instanciation-abbreviated' stamp: 'kph 5/10/2007 01:29'!
impara

	^ self monticello http: 'source.impara.de'! !

!Installer class methodsFor: 'instanciation' stamp: 'kph 1/8/2007 07:42'!
install: webPageName

self webSearchPath isEmpty ifTrue: [ ^self error: 'search path not set' ].

^(self new package: webPageName) install.
! !

!Installer class methodsFor: 'instanciation' stamp: 'kph 5/24/2007 19:47'!
installFile: fileName
 
^ (self file: fileName) install.
! !

!Installer class methodsFor: 'instanciation' stamp: 'kph 1/8/2007 07:42'!
installUrl: urlString

^ (self url: urlString) install.
! !

!Installer class methodsFor: 'launcher support' stamp: 'kph 2/27/2007 18:57'!
launchFrom: launcher

	^self launchWith: launcher getParameters! !

!Installer class methodsFor: 'launcher support' stamp: 'kph 5/24/2007 19:59'!
launchHelp

^'path=/dir/*.txt          Specify a search path for the item to install
p=/dir1/*.txt;<url2>/    Multiple items delimited by ;
                         The page name is typically appended to the path string, or
                         if a "*" is present, it will be replaced by the page name.
					
in,i,install=<page>      Page appended to the path to begin the install process
url,u=<url>              Install using an explicit url from which to obtain a script or file
file=<url>                Install using a local file
+debug                   Do not trap errors
view=<page>              Print the script that would have been installed.

For more options use Script eval="Installer ... " 
'
! !

!Installer class methodsFor: 'launcher support' stamp: 'kph 12/16/2007 12:04'!
launchWith: params

 	params at: 'P' ifPresent: [ :v | params at: 'PATH' put: v ].
 	params at: 'I' ifPresent: [ :v | params at: 'INSTALL' put: v ].
 	params at: 'IN' ifPresent: [ :v | params at: 'INSTALL' put: v ].
 	params at: 'U' ifPresent: [ :v | params at: 'URL' put: v ].

	params at: 'PATH' ifPresent: [ :v | 
		self webSearchPathFrom: v.
	].

	params at: 'USER' ifPresent: [ :v | 
		Utilities setAuthorInitials: v
	].

	params at: 'VIEW' ifPresent: [ :v |
		self view: v
	].

	IsSetToTrapErrors := true.
	params at: 'DEBUG' ifPresent: [ :v | IsSetToTrapErrors := (v == true) not ].

 	params at: 'URL' ifPresent: [ :v | 
		 self installUrl: v
	].

	params at: 'FILE' ifPresent: [ :v | 
		 self installFile: v
	].
 
 	params at: 'INSTALL' ifPresent: [ :v | 
		  self install: v
	].
 
	 ^true

	! !

!Installer class methodsFor: 'instanciation-abbreviated' stamp: 'kph 5/10/2007 01:33'!
lukas

	^ self monticello http: 'mc.lukas-renglli.ch'! !

!Installer class methodsFor: 'instanciation' stamp: 'kph 3/14/2007 21:07'!
mantis

	^ self mantis: 'http://bugs.squeak.org/'! !

!Installer class methodsFor: 'instanciation' stamp: 'kph 12/21/2006 04:26'!
mantis: host

^self new	ma: host; 
			markers: '&quot;fix begin&quot;...&quot;fix test&quot;...&quot;fix end&quot;'; 
			yourself.
! !

!Installer class methodsFor: 'instanciation' stamp: 'kph 5/10/2007 00:49'!
mc

	^ self new! !

!Installer class methodsFor: 'documentation' stamp: 'kph 5/24/2007 18:54'!
mczInstall: urlOrFile

	^ self new mczInstall: urlOrFile
! !

!Installer class methodsFor: 'instanciation' stamp: 'kph 5/10/2007 00:49'!
monticello

	^ self new! !

!Installer class methodsFor: 'debug' stamp: 'kph 5/9/2007 23:05'!
noDebug

	IsSetToTrapErrors := true! !

!Installer class methodsFor: 'instanciation' stamp: 'kph 1/16/2007 19:28'!
path: aString
	"convenience abbreviation"
	self webSearchPathFrom: aString! !

!Installer class methodsFor: 'accessing system' stamp: 'kph 5/28/2007 00:54'!
remembered

	^	Remembered ifNil: [ Remembered := IdentityDictionary new ]! !

!Installer class methodsFor: 'instanciation' stamp: 'kph 5/10/2007 01:05'!
repository: host  

^self monticello http: host ! !

!Installer class methodsFor: 'instanciation-abbreviated' stamp: 'kph 5/10/2007 01:21'!
sf

	^ self squeakfoundation
 ! !

!Installer class methodsFor: 'accessing' stamp: 'kph 12/21/2006 01:49'!
skipLoadingTests

"sets a flag to ignore loading of the testing portion of scripts embedded in pages"
SkipLoadingTests := true.
 ! !

!Installer class methodsFor: 'as yet unclassified' stamp: 'kph 1/4/2007 06:42'!
skipLoadingTests: yesNo during: block

	| oldValue |

	oldValue := SkipLoadingTests.
	SkipLoadingTests := yesNo.
	
	block ensure:[ SkipLoadingTests := oldValue ].! !

!Installer class methodsFor: 'instanciation-abbreviated' stamp: 'kph 5/10/2007 01:19'!
sm

	^ self squeakmap! !

!Installer class methodsFor: 'accessing system' stamp: 'kph 1/5/2007 02:50'!
smalltalkImage

^ SmalltalkImage current! !

!Installer class methodsFor: 'instanciation-abbreviated' stamp: 'kph 12/15/2007 11:08'!
sophie

	^ self monticello http: 'source.sophieproject.org'
	
! !

!Installer class methodsFor: 'accessing system' stamp: 'kph 1/5/2007 02:53'!
sourceFiles

^ SourceFiles! !

!Installer class methodsFor: 'instanciation-abbreviated' stamp: 'kph 5/10/2007 01:21'!
squeakfoundation

	^ self monticello http: 'source.squeakfoundation.org'! !

!Installer class methodsFor: 'instanciation' stamp: 'kph 10/15/2006 06:37'!
squeakmap

^self new sm: true; yourself! !

!Installer class methodsFor: 'instanciation-abbreviated' stamp: 'kph 5/10/2007 01:19'!
squeaksource

	^ self monticello http: 'www.squeaksource.com'! !

!Installer class methodsFor: 'instanciation-abbreviated' stamp: 'kph 5/10/2007 01:19'!
ss

	^ self squeaksource
 ! !

!Installer class methodsFor: 'instanciation' stamp: 'kph 7/25/2007 13:09'!
universe

	^ self new universe: (self classUGlobalInstaller universe: UUniverse systemUniverse)! !

!Installer class methodsFor: 'unload' stamp: 'kph 1/10/2007 15:27'!
unload: categoryMatchesString 

^ self new unload: categoryMatchesString! !

!Installer class methodsFor: 'instanciation' stamp: 'kph 12/15/2007 11:26'!
upgrade

	Installer ss project: 'Installer'; install: 'Installer-Core'.
	
	^ self! !

!Installer class methodsFor: 'instanciation' stamp: 'kph 11/10/2006 00:48'!
url: urlString

^self new url: urlString; yourself! !

!Installer class methodsFor: 'instanciation-abbreviated' stamp: 'kph 5/25/2007 01:34'!
validationBlock

	 
	^ ValidationBlock! !

!Installer class methodsFor: 'instanciation-abbreviated' stamp: 'kph 5/25/2007 01:34'!
validationBlock: aBlock

	ValidationBlock := aBlock! !

!Installer class methodsFor: 'instanciation' stamp: 'kph 1/8/2007 08:37'!
view: webPageNameOrUrl

| theReport |

theReport := String streamContents: [ :report | 
	(webPageNameOrUrl beginsWith: 'http://') ifTrue: [ 
		self new urlAction: ('Installer installUrl: ', (webPageNameOrUrl printString),'.')  	
						 reportOn: report.
	]
	ifFalse: [
		self new webAction: ('Installer install: ', (webPageNameOrUrl printString),'.')  	
						 reportOn: report.
	].
].

Workspace new contents: (theReport contents); openLabel: webPageNameOrUrl.

^theReport contents
! !

!Installer class methodsFor: 'instanciation' stamp: 'kph 5/9/2007 22:04'!
webInstall: webPageName

self webSearchPath isEmpty ifTrue: [ ^self error: 'search path not set' ].

^(self new package: webPageName) install.
! !

!Installer class methodsFor: 'accessing' stamp: 'kph 1/5/2007 00:18'!
webSearchPath

"a search path item, has the following format. prefix*suffix"

^ WebSearchPath ifNil: [ WebSearchPath := OrderedCollection new ].! !

!Installer class methodsFor: 'accessing' stamp: 'kph 12/15/2007 14:58'!
webSearchPathFrom: string

	| reader wsp path |
	
	reader := string readStream.
	wsp := self webSearchPath.
	
	[ reader atEnd ] whileFalse: [ 
		path := reader upTo: $;.
		(wsp includes: wsp) ifFalse: [ wsp addFirst: path ] 
	].

	 ! !

!Installer class methodsFor: 'instanciation' stamp: 'kph 10/21/2006 01:38'!
websqueakmap

^self new wsm: 'http://map.squeak.org'; yourself! !

!Installer class methodsFor: 'instanciation' stamp: 'kph 10/21/2006 01:38'!
websqueakmap: host

^self new wsm: host; yourself! !

!Installer class methodsFor: 'instanciation-abbreviated' stamp: 'kph 5/10/2007 01:30'!
wiresong

	^ self monticello http: 'source.wiresong.ca'! !

!Installer class methodsFor: 'instanciation-abbreviated' stamp: 'kph 5/10/2007 01:19'!
wsm

	^ self websqueakmap! !

!Installer methodsFor: 'action report' stamp: 'kph 1/11/2007 00:22'!
actionMatch: theLine reportOn: report ifNoMatch: aBlock

	| line |
	
	line := theLine withBlanksCondensed.
	
	((line beginsWith: 'Installer install:') | (line beginsWith: 'Installer do:')) 
		ifTrue: [ ^self webAction: theLine reportOn: report ].
	
	
	((line beginsWith: 'Installer installUrl:') and: 
		[ | ext |
		 ext :=  (line readStream upToAll: '''.') copyAfterLast: $..
		 (#( 'cs' 'st' 'mcz' 'sar') includes: ext) not ]) ifTrue: [ ^self urlAction: theLine reportOn: report ].

	(line beginsWith: 'Installer mantis fixBug:') ifTrue: [ ^self mantisAction: theLine reportOn: report ].
	
	
	aBlock value.
! !

!Installer methodsFor: 'auto answering' stamp: 'kph 10/17/2006 13:07'!
answer: aString with: anAnswer

^self answers add: ( Array with: aString with: anAnswer )! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/17/2006 13:06'!
answers

^ answers ifNil: [ answers := OrderedCollection new ]! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/19/2006 06:55'!
answers: anObject
	"Set the value of answers"

	answers := anObject! !

!Installer methodsFor: 'script bindings' stamp: 'kph 5/8/2007 19:52'!
bindingOf: aString 
	
	InstallerBindings isNil ifTrue: [ InstallerBindings := Dictionary new].

	(InstallerBindings includesKey: aString)
		ifFalse: [InstallerBindings at: aString put: nil].

	^ InstallerBindings associationAt: aString.! !

!Installer methodsFor: 'accessing' stamp: 'kph 5/24/2007 19:52'!
browse

self logErrorDuring: [
	self file ifNotNil: [ ^ self fileBrowse ].
	self mc ifNotNil: [ ^self mcBrowse ].
	self wsm ifNotNil: [ ^self wsmBrowse ].
	self sm ifTrue: [ ^self smBrowse ].
	self url ifNotNil: [ ^self urlBrowse ].
	self package ifNotNil: [ ^self webBrowse ].
] ! !

!Installer methodsFor: 'accessing' stamp: 'kph 5/9/2007 22:36'!
browse: packageNameCollectionOrDetectBlock

self package: packageNameCollectionOrDetectBlock.

self browse! !

!Installer methodsFor: 'mantis' stamp: 'kph 5/10/2007 00:15'!
browse: aFileName from: stream
	
	| mcThing |
		 
	self log: ' browsing...'.
 
	self classMCReader ifNil: [ ^ self browseNonMC: aFileName from: stream ].
		
	mcThing := 	self mcThing: aFileName from: stream.
	
	(mcThing respondsTo: #snapshot) ifTrue: [ ^ mcThing browse ].

  	(MCSnapshotBrowser forSnapshot: mcThing)
		showLabelled: 'Browsing ', aFileName

				 ! !

!Installer methodsFor: 'accessing' stamp: 'kph 11/7/2006 05:06'!
bug
	"Answer the value of bug"

	^ bug! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 09:48'!
bug: aBugNo

"
Installer mantis viewBug: 5639.
"
	| page text | 

	self setBug: aBugNo.
	
	page := self maPage.
 
	text := String streamContents: [ :str |	
			
		#('Bug ID' 'Category' 'Severity' 'Reproducibility' 'Date Submitted' 
			'Date Updated' 'Reporter' 'View Status' 'Handler' 
			'Priority' 'Resolution' 'Status' 'Product Version' 'Summary' 'Description' 'Additional Information' ) 
				do: [ :field |
						| f |
						f := self maRead: page field: field.
			str nextPutAll: f key; nextPutAll: ': '; nextPutAll: f value; cr.
		].
	
	str nextPutAll: 'Notes: '; cr.
		(self maReadNotes: page) do: [ :note | str nextPutAll: note; cr; cr ].
		
		str nextPutAll: 'Files: '; nextPutAll: self maFiles keys asArray printString.
	].
 	
^ text	
! !

!Installer methodsFor: 'mantis' stamp: 'kph 5/10/2007 00:16'!
bug: aBugNo browse: aFileName

	 self setBug: aBugNo.

	^ self browse: aFileName from: (self maThing: aFileName date: nil)! !

!Installer methodsFor: 'mantis' stamp: 'kph 5/25/2007 01:17'!
bug: aBugNo cl: aFileName

	| aFile list |
	
	self setBug: aBugNo.
	
	aFile :=  self maStreamForFile: aFileName.

	list := self classChangeList new
			scanFile:  aFile from: 1 to: aFile size.
		 
	self classChangeList open: list name: (aFileName, ' mantis: ', aBugNo printString) 
		multiSelect: true.! !

!Installer methodsFor: 'mantis' stamp: 'kph 12/18/2006 04:11'!
bug: aBugNo fix: aFileName

	^ self bug: aBugNo fix: aFileName date: nil! !

!Installer methodsFor: 'mantis' stamp: 'kph 5/11/2007 07:15'!
bug: aBugNo fix: aFileName date: aDate
 
	| fixed |

	self setBug: aBugNo.

	self ditchOldChangeSetFor: aFileName.

	self install: aFileName from: (self maThing: aFileName date: aDate).

	fixed := self class fixesApplied.
	(fixed isEmpty or: [ fixed last ~= aBugNo]) ifTrue: [ fixed add: aBugNo ].
		
	^ date! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 09:49'!
bug: aBugNo retrieve: aFileName

	 self setBug: aBugNo.

	^ (self maStreamForFile: aFileName) contents! !

!Installer methodsFor: 'mantis' stamp: 'kph 5/10/2007 00:34'!
bug: aBugNo view: aFileName
	" Installer mantis bug: 6089 browse: 'TTFSpeedUp-dgd.1.cs'. "
	 self setBug: aBugNo.

	^ self view: aFileName from: (self maThing: aFileName date: nil)! !

!Installer methodsFor: 'mantis' stamp: 'kph 9/29/2007 17:49'!
bugFiles: aBugNo
 
 	"provide a list of files associated with the bug in id order"
	"
	Installer mantis bugFiles: 6660.
	"
	self setBug: aBugNo.
	
	^ (self maFiles associations asSortedCollection: [ :a :b | a value asInteger < b value asInteger ]) collect: [ :a | a key ]! !

!Installer methodsFor: 'utils' stamp: 'kph 5/24/2007 19:16'!
changeSetNamed: aName

	(ChangeSet respondsTo: #named:)
		ifTrue: [ ^ChangeSet named: aName ].
		
	^ ChangeSorter changeSetNamed: aName.! !

!Installer methodsFor: 'class references' stamp: 'kph 12/23/2006 02:05'!
classChangeList

^Smalltalk at: #ChangeList  ifAbsent: [ self error: 'ChangeList not present' ]! !

!Installer methodsFor: 'class references' stamp: 'kph 11/16/2006 22:17'!
classChangeSet

^Smalltalk at: #ChangeSet  ifAbsent: [ self error: 'ChangeSet not present' ]! !

!Installer methodsFor: 'class references' stamp: 'kb 12/18/2006 12:40'!
classChangeSorter

^Smalltalk at: #ChangeSorter ifAbsent: [ self error: 'ChangeSorter not present' ]! !

!Installer methodsFor: 'class references' stamp: 'kph 11/16/2006 22:16'!
classGZipReadStream

^Smalltalk at: #GZipReadStream  ifAbsent: [ self error: 'Compression not present' ]! !

!Installer methodsFor: 'class references' stamp: 'kph 11/16/2006 15:42'!
classHTTPSocket

^Smalltalk at: #HTTPSocket ifAbsent: [ self error: 'Network package not present' ]! !

!Installer methodsFor: 'class references' stamp: 'kph 5/10/2007 01:01'!
classMCDirectoryRepository

	^Smalltalk at: #MCDirectoryRepository ifAbsent: [ self error: 'Monticello not present' ]
	! !

!Installer methodsFor: 'class references' stamp: 'kph 5/10/2007 00:56'!
classMCFtpRepository

	^Smalltalk at: #MCFtpRepository ifAbsent: [ self error: 'Monticello not present' ]
	! !

!Installer methodsFor: 'class references' stamp: 'kph 5/10/2007 00:59'!
classMCGOODSRepository

	^Smalltalk at: #MCGOODSRepository ifAbsent: [ self error: 'Monticello not present' ]
	! !

!Installer methodsFor: 'class references' stamp: 'kph 5/10/2007 00:56'!
classMCHttpRepository

	^Smalltalk at: #MCHttpRepository ifAbsent: [ self error: 'Monticello not present' ]
	! !

!Installer methodsFor: 'class references' stamp: 'kph 5/10/2007 00:59'!
classMCMagmaRepository

	^Smalltalk at: #MCMagmaRepository ifAbsent: [ self error: 'Magma not present' ]
	! !

!Installer methodsFor: 'class references' stamp: 'kph 11/17/2006 02:46'!
classMCMczReader

^Smalltalk at: #MCMczReader ifAbsent: [ nil ]
	! !

!Installer methodsFor: 'class references' stamp: 'kph 5/9/2007 18:12'!
classMCReader

^Smalltalk at: #MCReader ifAbsent: [ nil ]
	! !

!Installer methodsFor: 'class references' stamp: 'kph 5/10/2007 00:56'!
classMCSmtpRepository

	^Smalltalk at: #MCSmtpRepository ifAbsent: [ self error: 'Monticello not present' ]
	! !

!Installer methodsFor: 'class references' stamp: 'kph 11/17/2006 02:44'!
classMczInstaller

^Smalltalk at: #MczInstaller ifAbsent: [ nil ]
	! !

!Installer methodsFor: 'class references' stamp: 'kph 11/23/2006 03:18'!
classMultiByteBinaryOrTextStream
^Smalltalk at: #MultiByteBinaryOrTextStream  ifAbsent: [ self error: 'MultiByteBinaryOrTextStream not present' ]! !

!Installer methodsFor: 'class references' stamp: 'kph 11/16/2006 15:40'!
classSARInstaller

^Smalltalk at: #SARInstaller  ifAbsent: [ self error: 'SARInstaller not present' ]! !

!Installer methodsFor: 'class references' stamp: 'kph 2/1/2007 20:02'!
classSMLoader

^Smalltalk at: #SMLoader  ifAbsent: [ self error: 'SqueakMap Loader not present' ]! !

!Installer methodsFor: 'class references' stamp: 'kph 11/16/2006 15:40'!
classSMSqueakMap

^Smalltalk at: #SMSqueakMap  ifAbsent: [ self error: 'SqueakMap not present' ]! !

!Installer methodsFor: 'class references' stamp: 'kph 7/25/2007 13:09'!
classUVersion

^Smalltalk at: #UVersion  ifAbsent: [ self error: 'Universes code not present' ]! !

!Installer methodsFor: 'mantis' stamp: 'edc 4/4/2007 07:56'!
createRBforBug: aBugNo 
	| aStream  fileList selFile aFileName |

	self setBug: aBugNo.
fileList := self maFiles keys asOrderedCollection.
fileList  addLast: 'none'.
ReleaseBuilderFor3dot10 clear.
[selFile := UIManager default chooseFrom: fileList title: 'Choose what files load '.
selFile = fileList size ifFalse:[
aFileName := fileList at: selFile.
	self logCR: 'obtaining ', aFileName, '...'.

	aStream := self maStreamForFile: aFileName .
	ReleaseBuilderFor3dot10 current packagesInfluenced: aStream named: aFileName.
	
	self installCS: aFileName from: aStream].selFile = fileList size]whileFalse.
	
	ReleaseBuilderFor3dot10 current newUpdateFor: aBugNo
	
	
	! !

!Installer methodsFor: 'accessing' stamp: 'kph 12/21/2006 02:42'!
date: anObject
	"Set the value of markers"

	date := anObject asDate! !

!Installer methodsFor: 'accessing' stamp: 'kph 1/8/2007 07:22'!
desc
	 
	^ desc! !

!Installer methodsFor: 'accessing' stamp: 'kph 1/8/2007 07:22'!
desc: anObject

	desc := anObject! !

!Installer methodsFor: 'monticello' stamp: 'kph 5/10/2007 02:25'!
directory: dir

	mc := (self classMCDirectoryRepository new) directory:  dir; yourself.
 	root := dir! !

!Installer methodsFor: 'utils' stamp: 'kph 5/25/2007 01:12'!
ditchOldChangeSetFor: aFileName 

	| changeSetName |
 
	changeSetName := (self validChangeSetName: aFileName) sansPeriodSuffix.


	(self changeSetNamed: changeSetName)
		ifNotNil: [  	
				(self logCR:'Removing old change set ', changeSetName) cr.
				self  removeChangeSet: (self classChangeSet named: changeSetName) ].! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 10:10'!
ensureFix: aBugNo

	| fixesAppliedNumbers |
	
	self setBug: aBugNo.
	
	fixesAppliedNumbers := self class fixesApplied collect: [ :fixDesc | fixDesc asInteger ].
	
	(fixesAppliedNumbers includes: bug) ifFalse: [ self fixBug: aBugNo ]! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/7/2007 16:18'!
ensureFixes: aBugNos

	aBugNos do: [ :bugNo | self ensureFixes: bugNo ].! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 06:40'!
evaluate: stream

stream fileIn.! !

!Installer methodsFor: 'action report' stamp: 'kph 1/8/2007 11:14'!
extractFromHtml: html option: allOrLast

	|  start stop test in |

	start := self markersBegin.
	stop :=  self markersEnd.
	test := self markersTest.
			 
	in := ReadWriteStream with: String new.
		
	[ html upToAll: start; atEnd ] 
		whileFalse: [
			| chunk |
			(allOrLast == #last) ifTrue: [ in resetToStart ]. 
			chunk := html upToAll: stop.
			self isSkipLoadingTestsSet ifTrue: [ chunk := chunk readStream upToAll: test ].
			in nextPutAll: chunk. 
		 ].

	^self removeHtmlMarkupFrom: in reset
	 
! !

!Installer methodsFor: 'accessing' stamp: 'kph 7/28/2007 23:47'!
file

 ^ afile! !

!Installer methodsFor: 'accessing' stamp: 'kph 7/28/2007 23:48'!
file: f

  afile := f! !

!Installer methodsFor: 'file' stamp: 'kph 7/28/2007 23:48'!
fileBrowse
	
	self browse: self file from:  (FileDirectory readOnlyFileNamed: self file).

	! !

!Installer methodsFor: 'file' stamp: 'kph 7/28/2007 23:48'!
fileInstall
	
	self install: self file from: (FileDirectory default readOnlyFileNamed: self file)

	! !

!Installer methodsFor: 'file' stamp: 'kph 7/28/2007 23:48'!
fileView
	
	self view: self file from:  (FileDirectory readOnlyFileNamed: self file).

	! !

!Installer methodsFor: 'mantis' stamp: 'kph 12/18/2006 04:12'!
fixBug: aBugNo

	^ self fixBug: aBugNo date: nil! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 11:45'!
fixBug: aBugNo date: aDate

	self setBug: aBugNo.
	
 	self install: self maUrl from: self maScript.
	
	self maCheckDateAgainst: aDate.
	
	
	
! !

!Installer methodsFor: 'monticello' stamp: 'kph 5/10/2007 03:45'!
ftp: host directory: dir user: name password: secret
	"Installer mc ftp: 'mc.gjallar.se' directory: '' user: 'gjallar' password: secret. "
	mc := self classMCFtpRepository host: host directory: dir user: name password: ''.
	root :=  dir.	
 ! !

!Installer methodsFor: 'monticello' stamp: 'kph 5/10/2007 00:58'!
goods: host port: aport
	
	mc := (self classMCGOODSRepository new) host: host port: aport; yourself
 ! !

!Installer methodsFor: 'monticello' stamp: 'kph 12/15/2007 11:37'!
http: aUrl  
	
	self http: aUrl user: 'squeak' password: 'squeak'
		
 ! !

!Installer methodsFor: 'monticello' stamp: 'kph 5/10/2007 03:45'!
http: aUrl user: name password: secret
	
	mc := self classMCHttpRepository location: aUrl user: name password: secret.
	root := mc locationWithTrailingSlash	
 ! !

!Installer methodsFor: 'utils' stamp: 'kph 12/18/2006 06:50'!
httpGet: aUrl

	| page |

	page := self classHTTPSocket httpGet: aUrl accept: 'application/octet-stream'.  
 
	(page respondsTo: #reset)  ifFalse: [ self error: 'unable to contact web site' ].
 
	^ page
	! !

!Installer methodsFor: 'accessing' stamp: 'kph 11/12/2006 16:16'!
info

self sm ifTrue: [ ^ self smInfo  ].
self wsm ifNotNil: [ ^ self wsmInfo  ].! !

!Installer methodsFor: 'accessing' stamp: 'kph 7/25/2007 12:53'!
install

self logErrorDuring: [
	self universe ifNotNil: [ ^ self uniInstall ].
	self file ifNotNil: [ ^ self fileInstall ].
	self mc ifNotNil: [ ^self mcInstall ].
	self wsm ifNotNil: [ ^self wsmInstall ].
	self sm ifTrue: [ ^self smInstall ].
	self url ifNotNil: [ ^self urlInstall ].
	self package ifNotNil: [ ^self webInstall ].
] ! !

!Installer methodsFor: 'accessing' stamp: 'kph 5/25/2007 01:39'!
install: packageNameCollectionOrDetectBlock

self package: packageNameCollectionOrDetectBlock.
 
self install! !

!Installer methodsFor: 'mantis' stamp: 'kph 5/9/2007 23:07'!
install: aFileName from: stream

| ext installSelector mcThing |
	 
	self log: ' installing...'.
 
	self withAnswersDo:	[
		
		mcThing := self classMCReader ifNotNil: [ self mcThing: aFileName from: stream ].
		
		mcThing 
			ifNotNil: [ (mcThing respondsTo: #install) 
						ifTrue: [ mcThing install ]
				        	ifFalse: [ (mcThing respondsTo: #load) ifTrue: [ mcThing load ] ]
			]
			ifNil: [ 
		
				ext := aFileName copyAfterLast: $..
				installSelector := ('install', ext asUppercase, ':from:') asSymbol.
	
				(self respondsTo: installSelector)
					ifTrue: [ self perform: installSelector with: aFileName with: stream ]
					ifFalse: [ self installDefault: aFileName from: stream ].
			]
	]. 

	self log: ' done.'
! !

!Installer methodsFor: 'mantis' stamp: 'kph 5/9/2007 18:14'!
installCS: aFileName from: stream

 	self ditchOldChangeSetFor: aFileName.
	self newChangeSetFromStream: stream named: (self validChangeSetName: aFileName).
! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 06:42'!
installDefault: aFileName from: stream

stream fileIn.! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 06:42'!
installGZ: aFileName from: stream 

	"FileIn the contents of a gzipped stream"

	| zipped unzipped |
	 
	zipped := self classGZipReadStream on: stream.
	unzipped := MultiByteBinaryOrTextStream with: zipped contents asString.
	unzipped reset.

	self  newChangeSetFromStream: unzipped named:aFileName.

	! !

!Installer methodsFor: 'mantis' stamp: 'kph 5/9/2007 18:17'!
installMCZ: aFileName from: stream 

	self classMczInstaller ifNotNilDo: [ :reader | ^reader installStream: stream].  

	self error: 'no monticello readers available'. 
 ! !

!Installer methodsFor: 'mantis' stamp: 'kph 5/9/2007 18:07'!
installMCcs: aFileName from: stream 

	| reader |
	
	reader := MCCsReader on: stream.! !

!Installer methodsFor: 'mantis' stamp: 'kph 5/25/2007 01:09'!
installSAR: aFileName from: stream 

 | newCS |

	self classSARInstaller withCurrentChangeSetNamed: aFileName
		do: [:cs | newCS := cs. self classSARInstaller new fileInFrom: stream].
	newCS isEmpty ifTrue: [ self removeChangeSet: newCS ]! !

!Installer methodsFor: 'url' stamp: 'kph 1/10/2007 04:50'!
isHtmlStream: page

	"matches  '<!!DOCTYPE HTML', and <html>' "
	
	| first |	
	
	first := (page next: 14) asUppercase.
	
	^ (first = '<!!DOCTYPE HTML') | (first beginsWith: '<HTML>')
	
! !

!Installer methodsFor: 'accessing' stamp: 'kph 12/21/2006 02:07'!
isSkipLoadingTestsSet

^SkipLoadingTests ifNil: [ false ]! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/4/2007 06:43'!
justFixBug: aBugNo

	^self class skipLoadingTests: true during: [ self fixBug: aBugNo date: nil ]! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/4/2007 06:43'!
justFixBug: aBugNo date: d

	^self class skipLoadingTests: true during: [ self fixBug: aBugNo date: d ]! !

!Installer methodsFor: 'logging' stamp: 'kph 12/18/2006 04:51'!
log: text

^Transcript show: text.! !

!Installer methodsFor: 'logging' stamp: 'kph 5/25/2007 01:35'!
logCR: text

	self validate.
	
^ Transcript cr; show: text! !

!Installer methodsFor: 'logging' stamp: 'kph 7/31/2007 21:25'!
logErrorDuring: block

(IsSetToTrapErrors == false) ifTrue: [ ^ block value ].

block on: Error 
	do: [ :e | 
		
		self logCR: '****', e class name, ': ', (e messageText ifNil: [ '']). 
		
		e isResumable ifTrue:[ e resume: true ]
	
	]! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/19/2006 06:55'!
ma
	"Answer the value of ma"

	^ ma! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/19/2006 07:07'!
ma: aUrl
 
	"Set the value of host"

	ma := aUrl last = $/ ifTrue: [ aUrl ] ifFalse: [ aUrl, '/' ]! !

!Installer methodsFor: 'utils' stamp: 'kph 12/21/2006 02:51'!
maCheckDateAgainst: okDate

	(okDate notNil and: [date < okDate asDate ]) ifTrue: [ self notify: 'bug ', self bug asString, ' updated on ', date printString ].
 ! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 11:37'!
maFiles
 
 | file files bugPage id  | 
 
 files := Dictionary new.
 
 bugPage := self maPage.

 [
  id := bugPage upToAll: 'href="file_download.php?file_id='; upTo: $&. 
 file := bugPage upToAll: 'amp;type=bug"' ; upTo: $<.
 
 ((file size > 1) and: [file first = $>]) ifTrue: [ files at: file copyWithoutFirst put: id ].
 
 id notEmpty ] whileTrue.

^files ! !

!Installer methodsFor: 'mantis' stamp: 'kph 5/10/2007 03:09'!
maPage
  "
  self mantis bug: 5251.
 "
 | page |

page :=  self httpGet: self maUrl.
 
date := ((self maRead: page field: 'Date Updated') value upTo: $ ).

date isEmpty ifTrue: [ ^self error: bug, ' not found' ].

date := date asDate.
 
^page reset! !

!Installer methodsFor: 'mantis' stamp: 'kph 12/23/2006 04:50'!
maRead: page field: fieldKey

	 | value |
 
	value := page upToAll: ('!!-- ', fieldKey, ' -->'); upToAll: '<td'; upTo: $>; upToAll: '</td>'.
	
	page upTo: $<.
	
	page peek = $t ifTrue: [ value := page upToAll: 'td'; upTo: $>; upToAll: '</td>' ].
	  
	^Association key: fieldKey value: value withBlanksTrimmed! !

!Installer methodsFor: 'mantis' stamp: 'kph 12/23/2006 05:54'!
maReadNotes: page 

	 |  notes note  |
 
	notes := OrderedCollection new.

	[ page upToAll: 'tr class="bugnote"'; upTo: $>.
	  page atEnd ]
		
	whileFalse: [ 
		note := (self removeHtmlMarkupFrom: (page upToAll: '</tr>') readStream) contents.
		note := note withBlanksCondensed.
		"note replaceAll: Character cr with: $ ."
		note replaceAll: Character lf with: Character cr.
		notes add: note  
	].
	
	^notes! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 10:01'!
maScript 

	^self extractFromHtml: self maPage option: #last
! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 09:48'!
maStreamForFile: aFileName

	| fileId  |

 	fileId :=  self maFiles at: aFileName ifAbsent: [ self error: aFileName, ' not found' ].

 	^ self httpGet: (self ma, 'file_download.php?file_id=' , fileId , '&type=bug').
	 ! !

!Installer methodsFor: 'mantis' stamp: 'kph 5/9/2007 22:31'!
maThing: aFileName date: aDate
 
	self logCR: 'obtaining ', aFileName, '...'.

	pageDataStream := self maStreamForFile: aFileName.

	self maCheckDateAgainst: aDate.

	^ pageDataStream
	! !

!Installer methodsFor: 'mantis' stamp: 'kph 1/8/2007 09:54'!
maUrl
 
	^ url := self ma, 'view.php?id=', bug asString
 ! !

!Installer methodsFor: 'mantis' stamp: 'kph 12/18/2006 04:08'!
maUrlFor: maBugNo
 
	^ url := self ma, 'view.php?id=', maBugNo asString 
 ! !

!Installer methodsFor: 'monticello' stamp: 'kph 5/10/2007 00:58'!
magma: host port: aport
	
	mc := (self classMCMagmaRepository new) host: host port: aport; yourself
 ! !

!Installer methodsFor: 'action report' stamp: 'kph 1/8/2007 11:19'!
mantisAction: line reportOn: report
	
	| param mantis |
	
	mantis := Installer mantis.
	
	param :=  line readStream upTo: $: ; upTo: $..
	
	mantis setBug: ((param readStream upTo: $'; atEnd)
		 ifTrue: [  param ]
		 ifFalse: [ param readStream upTo: $'; upTo: $' ]).
	
	self reportSection: line on: report.
		
	report nextPutAll: (mantis replaceEntitiesIn: mantis markersBegin readStream).
	
	self reportFor: line page: mantis maScript on: report.
	
	report nextPutAll: (mantis replaceEntitiesIn: mantis markersEnd readStream); cr.
	! !

!Installer methodsFor: 'accessing' stamp: 'kph 1/4/2007 07:06'!
markers
	"Answer the value of markers"

	^ markers ifNil: [ '<code st>..."test ...</code st>' ]! !

!Installer methodsFor: 'accessing' stamp: 'kph 11/7/2006 05:06'!
markers: anObject
	"Set the value of markers"

	markers := anObject! !

!Installer methodsFor: 'url' stamp: 'kph 12/21/2006 01:54'!
markersBegin
		 	 
	 ^ self markers upTo: $.! !

!Installer methodsFor: 'url' stamp: 'kph 12/23/2006 04:02'!
markersEnd
	
	"return the third marker or the second if there are only two"
	
	| str  a |
		 	 
	 str := self markers readStream.
	 a := str upToAll: '...'; upToAll: '...'.
	 str atEnd  ifTrue: [ ^a ] ifFalse: [ ^str upToEnd ]
	! !

!Installer methodsFor: 'url' stamp: 'kph 12/21/2006 01:56'!
markersTest
		 	 
	 ^ self markers readStream upToAll: '...'; upToAll: '...'! !

!Installer methodsFor: 'searching' stamp: 'kph 1/4/2007 23:59'!
match: aMatch

	^self packagesMatching: aMatch! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 06:38'!
mc
	"Answer the value of mc"

	^ mc! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/19/2006 07:07'!
mc: aUrl
 
	"Set the value of host"

	mc := aUrl last = $/ ifTrue: [ aUrl ] ifFalse: [ aUrl, '/' ]! !

!Installer methodsFor: 'monticello' stamp: 'kph 5/9/2007 22:54'!
mcBrowse
	 "Installer ss project: 'Installer'; browse: 'Installer-Core'."
	 | it |
	 
	it := self mcThing.
	
	(it respondsTo: #browse) ifTrue: [ ^ it browse ].
	
	(MCSnapshotBrowser forSnapshot: it)
		showLabelled: 'Snapshot of ', self package! !

!Installer methodsFor: 'Installer-Core' stamp: 'kph 5/24/2007 19:49'!
mcDetectFileBlock

	self package isString ifTrue: [  ^ [ :aFile | aFile beginsWith: self package ] ].

	(self package isKindOf: Array) 
			ifTrue: [  ^  [ :aFile | (self package detect: [ :item | aFile beginsWith: item ] ifNone: [ false ]) ~= false ] ].

	self package isBlock ifTrue: [ ^ self package ].
  
 ! !

!Installer methodsFor: 'monticello' stamp: 'kph 5/9/2007 19:11'!
mcInstall
	 
	self withAnswersDo: [ self mcThing load ].

	self log: 'loaded'.
! !

!Installer methodsFor: 'monticello' stamp: 'kph 11/16/2006 17:47'!
mcRepository

^self classMCHttpRepository location: (self mcUrl) user: self user password: self password.! !

!Installer methodsFor: 'monticello' stamp: 'kph 5/10/2007 01:42'!
mcSortFileBlock

	^ [:a :b | 
        	[(a findBetweenSubStrs: #($.)) allButLast last asInteger > (b findBetweenSubStrs: #($.)) allButLast last asInteger] on: Error do: [:ex | false]].! !

!Installer methodsFor: 'monticello' stamp: 'kph 5/10/2007 02:17'!
mcThing

	| files fileToLoad  version  count |

	self logCR: 'finding ', self package asString, '...'.
 

	"several attempts to read files - repository readableFileNames sometimes fails"

	count := 0. fileToLoad := nil.
	
	[count := count + 1.
	 (fileToLoad = nil) and:[ count < 5 ] ] 
		whileTrue: [
						files := mc readableFileNames asSortedCollection: self mcSortFileBlock.
						fileToLoad := files detect: self mcDetectFileBlock ifNone: [ nil ].
		].

	version := mc versionFromFileNamed: fileToLoad.
	version workingCopy repositoryGroup addRepository: mc.
	mc creationTemplate: mc asCreationTemplate.
	
	self log: ' found ', version fileName, '...'.

	^ version! !

!Installer methodsFor: 'mantis' stamp: 'abc 7/4/2007 01:32'!
mcThing: aFileName from: stream
		
	"dont use monticello for .cs or for .st use monticello for .mcs"

	| reader |
	
	reader := self classMCReader readerClassForFileNamed: aFileName.
	
	reader name = 'MCStReader' ifTrue: [ ^ nil ].
 
	reader ifNil: [ ^ nil ].
	
	reader := reader on: stream.
	
	(reader respondsTo: #version) 
		ifTrue: [ ^ reader version  ]
		ifFalse: [ ^ reader snapshot  ].! !

!Installer methodsFor: 'monticello' stamp: 'kph 12/17/2006 17:58'!
mcUrl

^ self mc, (self project ifNil: [''])! !

!Installer methodsFor: 'monticello' stamp: 'kph 10/21/2006 05:47'!
mcVersions

^ (self packages select: [ :p | ( self package,'-*.mcz' ) match: p ]) collect: [ :p | self copy package: p  ; yourself ].
 ! !

!Installer methodsFor: 'monticello' stamp: 'kph 5/9/2007 22:52'!
mcView
	 "Installer ss project: 'Installer'; view: 'Installer-Core'. "
	 | it |
	
	it := self mcThing. 
	
	(it respondsTo: #open) ifTrue: [ ^ it open ].

	(MCSnapshotBrowser forSnapshot: it)
		showLabelled: 'Snapshot of ', self package! !

!Installer methodsFor: 'monticello' stamp: 'kph 5/24/2007 18:56'!
mczInstall: urlOrFile

	self log: ('Loading ', urlOrFile, ' ...').

	(urlOrFile beginsWith: 'http:')
		ifTrue: [  MczInstaller installStream: (HTTPSocket httpGet: urlOrFile) ]
		ifFalse: [ MczInstaller installFileNamed: urlOrFile ].
		
	self logCR: ' Loaded'.

	

! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/17/2006 14:30'!
messagesToSuppress

^ messagesToSuppress ifNil: [ messagesToSuppress := OrderedCollection new ]! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/19/2006 06:55'!
messagesToSuppress: anObject
	"Set the value of messagesToSuppress"

	messagesToSuppress := anObject! !

!Installer methodsFor: 'mantis' stamp: 'kb 12/18/2006 13:01'!
newChangeSetFromStream: aStream named: aName 

	"This code is based upon ChangeSet-c-#newChangesFromStream:named: which is in 3.9,
	implemented here for previous versions. The second branch is for 3.8, where ChangeSets
	are loaded by ChangeSorter. "

	| oldChanges newName newSet newStream |

	(self classChangeSet respondsTo: #newChangesFromStream:named:) 
		ifTrue: [ ^self classChangeSet newChangesFromStream: aStream named:aName ].

	(self classChangeSorter respondsTo: #newChangesFromStream:named:)
		ifTrue: [ ^self classChangeSorter newChangesFromStream: aStream named: aName ].

	oldChanges := ChangeSet current.
 
	"so a Bumper update can find it"
	newName := aName sansPeriodSuffix.

	newSet := self classChangeSet basicNewNamed: newName.

	[newSet
		ifNotNil: [(aStream respondsTo: #converter:)
				ifTrue: [newStream := aStream]
				ifFalse: [newStream := self classMultiByteBinaryOrTextStream with: aStream contentsOfEntireFile.
					newStream reset].
			self classChangeSet newChanges: newSet.
			newStream setConverterForCode.
			newStream fileInAnnouncing: 'Loading ' , newName , '...'.
			Transcript cr; show: 'File ' , aName , ' successfully filed in to change set ' , newName].
	aStream close]
		ensure: [self classChangeSet newChanges: oldChanges].
	 
	^ newSet! !

!Installer methodsFor: 'accessing' stamp: 'kph 5/10/2007 02:35'!
open
	
self sm ifTrue: [ self classSMLoader open ].
self mc ifNotNil: [self mc morphicOpen: nil ].! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 04:05'!
package
	"Answer the value of package"

	^ package! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 04:05'!
package: anObject
	"Set the value of package"

	package := anObject! !

!Installer methodsFor: 'accessing' stamp: 'kph 5/10/2007 02:37'!
packages
	
self sm ifTrue: [ ^self smPackages ].
self mc ifNotNil: [ ^self mc allFileNames ].
self wsm ifNotNil: [ ^self wsmPackagesByName keys ].! !

!Installer methodsFor: 'accessing' stamp: 'kph 11/7/2006 05:06'!
packages: anObject
	"Set the value of packages"

	packages := anObject! !

!Installer methodsFor: 'searching' stamp: 'kph 11/12/2006 23:19'!
packagesMatching: aMatch

self sm ifTrue: [ ^ (self packages select: [ :p | aMatch match: p name ]) collect: [ :p | self copy package: p name; yourself ] ].
self mc ifNotNil: [ ^ (self packages select: [ :p | ( aMatch , '.mcz' ) match: p ]) collect: [ :p | self copy package: p ; yourself ] ].
self wsm ifNotNil: [ ^ (self packages select: [ :p | ( aMatch) match: p ]) collect: [ :p | self copy package: p ; yourself ] ].

^'search type not supported'! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 04:16'!
password
	"Answer the value of password"

	^ password  ifNil: [ '' ]! !

!Installer methodsFor: 'accessing' stamp: 'kph 5/10/2007 02:29'!
printOn: s
 
s nextPutAll: '(Installer'.

self sm ifTrue: [ s nextPutAll: ' squeakmap' ].
self ma ifNotNil: [ s nextPutAll: ' mantis' ].
self wsm ifNotNil: [ s nextPutAll: ' websqueakmap' ].
self url ifNotNil: [ s nextPutAll: ' url:''', self url,'''' ].
self mc ifNotNil: [ s nextPutAll: ' repository:''', mc description,'''' ].

s nextPut: $).

self project ifNotNil: [ s nextPutAll: ' project:';
  						nextPutAll: '''', self project, ''''.
					   self package ifNotNil: [ s nextPutAll: '; '] ].
					
self package ifNotNil: [ s nextPutAll: ' package:';
  						 nextPutAll: '''', self package asString, '''' ].
					
s nextPut: $..! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 06:30'!
project
	"Answer the value of project"

	^ project! !

!Installer methodsFor: 'accessing' stamp: 'kph 5/10/2007 02:13'!
project: name
	"Set the value of project"

	project := name.
	
	(mc respondsTo: #location:) ifTrue:[ mc := mc copy location: root , name ].
	(mc respondsTo: #directory:) ifTrue: [ mc := mc copy directory: root ,'/', name ].
		
	^self copy.! !

!Installer methodsFor: 'accessing' stamp: 'kph 5/28/2007 03:42'!
rememberAs: symbol

	self class remembered at: symbol asSymbol put: self! !

!Installer methodsFor: 'utils' stamp: 'kph 5/25/2007 01:15'!
removeChangeSet: cs

	(self classChangeSet respondsTo: #removeChangeSet:)
		ifTrue: [ ^ChangeSet removeChangeSet: cs ].
		
	^ self classChangeSorter removeChangeSet: cs .! !

!Installer methodsFor: 'url' stamp: 'kph 1/8/2007 11:16'!
removeHtmlMarkupFrom: in 

	| out |
 
	out := ReadWriteStream on: (String new: 100).
	[ in atEnd ] whileFalse: [ 
		out nextPutAll: (in upTo: $<).
		(((in upTo: $>) asLowercase beginsWith: 'br') and: [ (in peek = Character cr) ]) ifTrue: [ in next ].	
	].
	
	^self replaceEntitiesIn: out reset.
! !

!Installer methodsFor: 'url' stamp: 'kph 1/8/2007 11:15'!
replaceEntitiesIn: in

	| out |
	
	out := ReadWriteStream on: (String new: 100).
	[ in atEnd ] whileFalse: [ 
		out nextPutAll: ((in upTo: $&) replaceAll: Character lf with: Character cr).
		in atEnd ifFalse: [ out nextPutAll: (self class entities at: (in upTo: $;) ifAbsent: '?') ].	
	].

	^out reset! !

!Installer methodsFor: 'action report' stamp: 'kph 1/8/2007 08:17'!
reportFor: theLine page: thePage on: report 
 	
	[ thePage atEnd ] whileFalse: [ 
		| line |
		line := thePage nextLine.
		self actionMatch: line reportOn: report ifNoMatch: [ report nextPutAll: line; cr. ] 	
	].! !

!Installer methodsFor: 'action report' stamp: 'kph 1/8/2007 11:20'!
reportSection: line on: report
	
	report isEmpty ifFalse: [ report cr ].
	
	report nextPutAll: '">>>> ' ; nextPutAll: (line copyWithout: $"); nextPut: $"; cr.

	! !

!Installer methodsFor: 'searching' stamp: 'kph 11/12/2006 23:19'!
search: aMatch

self sm ifTrue: [ ^ self smSearch: ('*',aMatch,'*') ].
self mc ifTrue: [ ^ self mcSearch: ('*',aMatch,'*')  ].
self wsm ifNotNil: [ ^ self wsmSearch: ('*',aMatch,'*')  ].

^'search type not supported'! !

!Installer methodsFor: 'mantis' stamp: 'kph 5/10/2007 03:12'!
setBug: stringOrNumber
 
  | str |
 
 self logCR: stringOrNumber.
 
 stringOrNumber isInteger ifTrue: [ bug := stringOrNumber. desc := ''. ^self ].
 
 bug := stringOrNumber asInteger.
 str := str printString. 
 desc := stringOrNumber copyFrom: (str size + 1) to: (stringOrNumber size) 

! !

!Installer methodsFor: 'mantis' stamp: 'kph 12/21/2006 01:28'!
skipTests

! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 06:55'!
sm
	"Answer the value of sm"

	^ sm ifNil: [ false ]! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 06:38'!
sm: anObject
	"Set the value of sm"

	sm := anObject! !

!Installer methodsFor: 'squeakmap' stamp: 'kph 5/10/2007 00:29'!
smBrowse

	self smThing explore! !

!Installer methodsFor: 'squeakmap' stamp: 'kph 5/9/2007 21:51'!
smInstall 

	self log: ' installing '. 
	
	self withAnswersDo: [ self smThing install ].

	self log: ' done'.
! !

!Installer methodsFor: 'squeakmap' stamp: 'kph 10/17/2006 15:24'!
smPackageAndVersion

| p |

p := ReadStream on: self package .

^Array with: (p upTo: $() with: (p upTo: $)).! !

!Installer methodsFor: 'squeakmap' stamp: 'kph 2/1/2007 19:14'!
smPackages
	
^self classSMSqueakMap default packagesByName! !

!Installer methodsFor: 'squeakmap' stamp: 'kph 11/16/2006 15:40'!
smReleasesForPackage: name 

^(self classSMSqueakMap default packageWithName: name) releases! !

!Installer methodsFor: 'searching' stamp: 'kph 11/12/2006 23:28'!
smSearch: aMatch  

	| results |
	
	results := Set new.

	self packages do: [ :pkg |
		({ 'name:',pkg name.
		   'summary:', pkg summary.
		   'description:', pkg description.
		   'author:', pkg author. } anySatisfy: [ :field | aMatch match: field ])
		 ifTrue: [ results add: (self copy package: pkg name) ]. 
	].

	^results

! !

!Installer methodsFor: 'squeakmap' stamp: 'kph 5/9/2007 21:50'!
smThing 

	| pkgAndVersion releases release |

	pkgAndVersion := self smPackageAndVersion.

	self logCR: 'retrieving ', self package, ' from SqueakMap...'.

	releases := self smReleasesForPackage: pkgAndVersion first.
 	
	release := pkgAndVersion last isEmpty ifTrue: [ releases last ]
					ifFalse:[ releases detect: [ :rel | rel version = pkgAndVersion last ] ]. 

	^ release
		
	
! !

!Installer methodsFor: 'squeakmap' stamp: 'kph 11/12/2006 16:54'!
smVersions
 
^ (self smReleasesForPackage: self package) collect: [ :v | self copy package: (v package name,'(',v version,')'); yourself. ] 

 ! !

!Installer methodsFor: 'squeakmap' stamp: 'kph 5/10/2007 00:25'!
smView

	self smThing explore! !

!Installer methodsFor: 'auto answering' stamp: 'kph 10/17/2006 14:30'!
suppress: aMessage

messagesToSuppress add: aMessage! !

!Installer methodsFor: 'universes' stamp: 'kph 7/25/2007 13:46'!
uniDoInstall

	self withAnswersDo: [ self universe doInstall ] ! !

!Installer methodsFor: 'universes' stamp: 'kph 7/25/2007 13:46'!
uniInstall

	| pkgAndVersion pkg version potentials |
	
	package ifNil:[ ^ self uniDoInstall ].
	
	pkgAndVersion := self smPackageAndVersion.
	pkg := pkgAndVersion first.
	version := pkgAndVersion last.
	
	potentials := universe packageVersionsForPackage: pkg.
	
	pkg := version isEmpty 
			ifTrue: [ potentials last ]
			ifFalse: [ 
				version := self classUVersion readFrom: version.  
				potentials detect:[ :p | p version = version] ifNone: [ ^ self error: 'version not found']
			].
		
	universe planToInstallPackage: pkg.
	
	package := nil.! !

!Installer methodsFor: 'universes' stamp: 'kph 12/16/2007 12:36'!
uniUpdate

 
	(LastUniUpdate isNil or:[ (DateAndTime now - LastUniUpdate) > 600 seconds  ])
	ifTrue: [ 
		Utilities informUser: 'updating package list...' during:  [
			universe updatePackagesViaWWW ]
	].

	LastUniUpdate := DateAndTime now.! !

!Installer methodsFor: 'universes' stamp: 'kph 7/25/2007 12:52'!
universe

	^ universe! !

!Installer methodsFor: 'universes' stamp: 'squeak 7/26/2007 18:12'!
universe: u

	universe := u.
	self uniUpdate.! !

!Installer methodsFor: 'script bindings' stamp: 'kph 1/11/2007 02:03'!
unload: categoryMatchesString 

 (SystemOrganization categoriesMatching: categoryMatchesString) do: [ :cat | 
     self logCR: 'Unloading ', cat.
 	(MCPackage named: cat)  workingCopy unload.
	SystemOrganization removeCategory: cat.
 ].

"Until Mantis 5718 is addressed"
 Smalltalk at: #PackagePaneBrowser ifPresent: [ :ppbClass | ppbClass allInstancesDo: [ :ppb | ppb updatePackages ]  ].
 Smalltalk at: #Browser ifPresent: [ :bClass | bClass allInstancesDo: [ :b | b updateSystemCategories ] ].! !

!Installer methodsFor: 'accessing' stamp: 'kph 11/7/2006 04:23'!
url

^url! !

!Installer methodsFor: 'accessing' stamp: 'kph 11/7/2006 04:23'!
url: aUrl
 
 url := aUrl! !

!Installer methodsFor: 'action report' stamp: 'kph 1/8/2007 11:19'!
urlAction: line reportOn: report 
 
	url :=  line readStream upTo: $' ; upTo: $'.
  	
	self reportSection: line on: report.

	(pageDataStream := self urlGet: self urlToDownload) 
		ifNil: [ self error: 'unable to contact host' ].
	 	
	self reportFor: line page: pageDataStream on: report ! !

!Installer methodsFor: 'url' stamp: 'kph 5/9/2007 23:03'!
urlBrowse
 	"(Installer debug url: 'http://installer.pbwiki.com/f/Installer.st') browse. ".
	self browse: self urlToDownload from: self urlThing.
	
	
! !

!Installer methodsFor: 'url' stamp: 'kph 1/7/2007 02:15'!
urlGet

^ self urlGet: self urlToDownload! !

!Installer methodsFor: 'url' stamp: 'kph 1/10/2007 04:35'!
urlGet: aUrl

	| page |

	page := HTTPSocket httpGet: aUrl accept: 'application/octet-stream'.  
 
	(page respondsTo: #reset)  ifFalse: [ ^ nil ].
 
	(self isHtmlStream: page) ifTrue: [ page := self extractFromHtml: page option: nil ].

	^ page reset
	! !

!Installer methodsFor: 'url' stamp: 'kph 5/9/2007 21:58'!
urlInstall 
 	 
	self install: self urlToDownload from: self urlThing.
	
	^ pageDataStream 
! !

!Installer methodsFor: 'url' stamp: 'kph 5/9/2007 21:57'!
urlThing
 
	self logCR: 'retrieving ', self urlToDownload , ' ...'.
	
	(pageDataStream := self urlGet: self urlToDownload) 
		ifNil: [ self error: 'unable to contact host' ].
	 
	^ pageDataStream
	! !

!Installer methodsFor: 'url' stamp: 'kph 1/4/2007 23:49'!
urlToDownload

	^ (self url, (self package ifNil: [ '' ])) asUrl asString.
	
 ! !

!Installer methodsFor: 'url' stamp: 'kph 5/10/2007 00:19'!
urlView
 	 "(Installer debug url: 'http://installer.pbwiki.com/f/Installer.st') view. ".
	self view: self urlToDownload from: self urlThing.
	
	
! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 04:15'!
user
	"Answer the value of user"

	^ user ifNil: [ '' ]! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/15/2006 03:56'!
user: anObject
	"Set the value of user"

	user := anObject! !

!Installer methodsFor: 'url' stamp: 'kph 5/8/2007 12:27'!
validChangeSetName: aFileName
	" dots in the url confuses the changeset loader. I replace them with dashes"
 	(aFileName beginsWith:'http:') ifTrue: [ | asUrl |
		asUrl := Url absoluteFromText: aFileName.
		^String streamContents: [:stream |
			stream nextPutAll: (asUrl authority copyReplaceAll: '.' with: '-').
			asUrl path allButLastDo: [:each |
				stream
					nextPutAll: '/';
					nextPutAll: (each copyReplaceAll: '.' with: '-') ].
			stream
				nextPutAll: '/';
				nextPutAll: asUrl path last ] ].
	^aFileName! !

!Installer methodsFor: 'logging' stamp: 'kph 5/25/2007 01:36'!
validate

	ValidationBlock value = false ifTrue: [ self error: 'Validation failed' ].! !

!Installer methodsFor: 'accessing' stamp: 'kph 10/21/2006 05:50'!
versions
	
self sm ifTrue: [ ^self smVersions ].
self mc ifNotNil: [ ^self mcVersions ].
self wsm ifNotNil: [ ^self wsmVersions ].! !

!Installer methodsFor: 'accessing' stamp: 'kph 5/24/2007 19:52'!
view

self logErrorDuring: [
	self file ifNotNil: [ ^ self fileView ].
	self mc ifNotNil: [ ^self mcView ].
	self wsm ifNotNil: [ ^self wsmView ].
	self sm ifTrue: [ ^self smView ].
	self url ifNotNil: [ ^self urlView ].
	self package ifNotNil: [ ^self webView ].
] ! !

!Installer methodsFor: 'accessing' stamp: 'kph 5/9/2007 21:44'!
view: packageNameCollectionOrDetectBlock

self package: packageNameCollectionOrDetectBlock.

self view! !

!Installer methodsFor: 'mantis' stamp: 'kph 5/10/2007 00:21'!
view: aFileName from: stream
	
	self log: ' viewing...'.
 	
	Workspace new contents: (stream contents); openLabel: aFileName.
 
	 

				 ! !

!Installer methodsFor: 'mantis' stamp: 'kph 12/23/2006 06:02'!
viewBug: aBugNo

	^Workspace new contents: (self bug: aBugNo); openLabel: ('Mantis ', aBugNo printString).
! !

!Installer methodsFor: 'utils' stamp: 'kph 5/9/2007 21:49'!
viewUrl

	^Workspace new contents: (self urlGet contents); openLabel: self urlToDownload.
! !

!Installer methodsFor: 'action report' stamp: 'kph 1/8/2007 11:19'!
webAction: line reportOn: report
	
	package :=  line readStream upTo: $' ; upTo: $'.

	self reportSection: line on: report.
	
 	url := self webFindUrlToDownload.
	
	self reportFor: line page: pageDataStream on: report ! !

!Installer methodsFor: 'web install' stamp: 'kph 5/24/2007 19:56'!
webBrowse
 
 	 self webThing size > 0 
		ifTrue: [ self browse: url from: pageDataStream ]
		ifFalse: [ self logCR: 'NO DATA ',url,' was empty' ].
	 ! !

!Installer methodsFor: 'web install' stamp: 'kph 12/16/2007 12:01'!
webFindUrlToDownload

	self class webSearchPath do: [ :pathSpec | 
		| potentialUrl readPathSpec  |
	
		readPathSpec := pathSpec value readStream.
		potentialUrl := (readPathSpec upTo: $*), package, (readPathSpec upToEnd ifNil: [ '' ]).
	
		pageDataStream := self urlGet: potentialUrl.
		
		pageDataStream notNil ifTrue: [ ^potentialUrl ]
	].

	^nil
! !

!Installer methodsFor: 'web install' stamp: 'kph 5/9/2007 22:02'!
webInstall
 
 	 self webThing size > 0 
		ifTrue: [ self install: url from: pageDataStream ]
		ifFalse: [ self logCR: '...',url,' was empty' ].
	 ! !

!Installer methodsFor: 'web install' stamp: 'kph 5/9/2007 22:02'!
webThing
 
 	url := self webFindUrlToDownload ifNil: [ self logCR: 'url not found'. self error: 'url not found' ].
	
	self logCR: 'found ',  url, ' ...'.
	
	^ pageDataStream
	! !

!Installer methodsFor: 'web install' stamp: 'kph 5/9/2007 22:08'!
webView
 
 	 self webThing size > 0 
		ifTrue: [ self view: url from: pageDataStream ]
		ifFalse: [ self logCR: 'NO DATA ',url,' was empty' ].
	 ! !

!Installer methodsFor: 'monticello' stamp: 'kph 10/21/2006 06:52'!
withAnswersDo: aBlock

	(aBlock respondsTo: #valueSuppressingMessages:supplyingAnswers: )
		ifTrue: [aBlock valueSuppressingMessages: self messagesToSuppress supplyingAnswers: self answers.]
		ifFalse: [ aBlock value ]
! !

!Installer methodsFor: 'websqueakmap' stamp: 'kph 10/21/2006 01:37'!
wsm
	"Answer the value of wsm"
	
	^ wsm! !

!Installer methodsFor: 'websqueakmap' stamp: 'kph 10/21/2006 01:37'!
wsm: aUrl
 
	"Set the value of host"

	wsm := aUrl last = $/ ifTrue: [ aUrl ] ifFalse: [ aUrl, '/' ]! !

!Installer methodsFor: 'websqueakmap' stamp: 'kph 12/18/2006 06:49'!
wsmDownloadUrl 

| pkgAndVersion packageId packageName packageVersion releaseAutoVersion
 downloadPage |

	pkgAndVersion := self smPackageAndVersion.
	packageName := pkgAndVersion first.
	packageVersion := pkgAndVersion last.
	packageVersion isEmpty ifTrue: [ packageVersion := #latest ].

	packageId := self wsmPackagesByName at: packageName.
	releaseAutoVersion := (self wsmReleasesFor: packageId) at: packageVersion.
					 
	downloadPage := self httpGet: (self wsm,'packagebyname/', packageName,'/autoversion/', releaseAutoVersion,'/downloadurl') asUrl asString.
				 		 
	^ downloadPage contents
	
! !

!Installer methodsFor: 'websqueakmap' stamp: 'kph 5/25/2007 01:57'!
wsmInstall
	
	| it |
	
	it := self wsmThing.

	self install: it from: it asUrl retrieveContents contentStream.

	! !

!Installer methodsFor: 'websqueakmap' stamp: 'kph 12/18/2006 06:51'!
wsmPackagesByName

	| html id name | 
	
	packages ifNotNil: [ ^packages ].
	
	packages := Dictionary new.
	
	html := self httpGet: (self wsm, 'packagesbyname').
	
	[
		id := html upToAll: '/package/'; upToAll: '">'.
		name := html upTo: $<.
		
		(id notEmpty and: [ name notEmpty ]) 

	] whileTrue: [ packages at: name put: id ].

	^ packages	
	! !

!Installer methodsFor: 'websqueakmap' stamp: 'kph 12/18/2006 06:50'!
wsmReleasesFor: packageId

	| html autoVersion version releases |
		
	releases := Dictionary new.
	
	html := self httpGet: (self wsm, '/package/', packageId ).
	
	[
		releases at: #latest put: autoVersion.
		autoVersion := html upToAll: '/autoversion/'; upTo: $".
		version := html upTo: $-; upTo: $<.
		
		(autoVersion notEmpty and: [version notEmpty ]) 

	] whileTrue: [ releases at: version put: autoVersion ].

	^ releases
	! !

!Installer methodsFor: 'websqueakmap' stamp: 'kph 5/9/2007 21:55'!
wsmThing

	| downloadUrl |

	self logCR: 'finding ', self package, ' from websqueakmap(', self wsm, ') ...'.

	downloadUrl := self wsmDownloadUrl.
	
	self logCR: 'found at ', downloadUrl asString, ' ...'.
	 
	^ downloadUrl
	! !

!Installer methodsFor: 'websqueakmap' stamp: 'kph 10/21/2006 06:04'!
wsmVersions

| pkgAndVersion packageId packageName packageVersion versions |

	pkgAndVersion := self smPackageAndVersion.
	packageName := pkgAndVersion first.
	packageVersion := pkgAndVersion last.
	packageVersion isEmpty ifTrue: [ packageVersion := #latest ].

	packageId := self wsmPackagesByName at: packageName.
	
	versions := (self wsmReleasesFor: packageId) keys.
	versions remove: #latest.
	
	^ versions collect: [ :version | self copy package: (packageName,'(', version ,')'); yourself ]. ! !

!Installer methodsFor: 'websqueakmap' stamp: 'kph 5/9/2007 21:55'!
wsmView
	
	| it |
	
	it := self wsmThing.
	self view: it from: (self httpGet: it).

	! !

!SmalltalkImage class methodsFor: '*installer-core' stamp: 'kph 2/27/2007 18:57'!
launchFrom: launcher

 ^self launchWith: launcher getParameters! !

!SmalltalkImage class methodsFor: '*installer-core' stamp: 'kph 2/16/2007 10:05'!
launchHelp

^
'+save                    Save image
save=name.image          Save image with given file name
+quit                    Quit squeak
'! !

!SmalltalkImage class methodsFor: '*installer-core' stamp: 'kph 5/8/2007 12:07'!
launchWith: params

	| saveAs quit resuming |

	saveAs := params at: 'SAVE' ifAbsent: false.
	quit := params at: 'QUIT' ifAbsent: false.

	(saveAs isKindOf: Boolean) 
		ifFalse: [
				  (saveAs endsWith: FileDirectory imageSuffix) ifFalse: [ saveAs := saveAs, '.', FileDirectory imageSuffix ]. 
				  resuming := self current saveAs: saveAs.
				  resuming ifTrue: [ ^false ].
				  saveAs := false ].
		
 	resuming := self current snapshot: saveAs andQuit: (quit == true).
	 
	resuming ifTrue: [ ^false ].
	"returning false, stops evaluation of further commandline parameters
	 this will be beneficial for image restart"

	^ true
	! !
Installer initialize!
